/*!\file BeamVertexElementResult.c
 * \brief: implementation of the BeamVertexElementResult object
 */

#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include "stdio.h"
#include <string.h>
#include "./ElementResultLocal.h"
#include "../objects.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../shared/shared.h"
#include "../../Container/Container.h"
#include "../../include/include.h"

/*BeamVertexElementResult constructors and destructor*/
/*FUNCTION BeamVertexElementResult::BeamVertexElementResult(){{{1*/
BeamVertexElementResult::BeamVertexElementResult(){
	return;
}
/*}}}*/
/*FUNCTION BeamVertexElementResult::BeamVertexElementResult(int in_enum_type,double* in_values,int in_step, double in_time){{{1*/
BeamVertexElementResult::BeamVertexElementResult(int in_enum_type,double* in_values,int in_step, double in_time){

	enum_type=in_enum_type;
	values[0]=in_values[0];
	values[1]=in_values[1];
	step=in_step;
	time=in_time;
}
/*}}}*/
/*FUNCTION BeamVertexElementResult::~BeamVertexElementResult(){{{1*/
BeamVertexElementResult::~BeamVertexElementResult(){
	return;
}
/*}}}*/

/*Object virtual functions definitions:*/
/*FUNCTION BeamVertexElementResult::Echo {{{1*/
void BeamVertexElementResult::Echo(void){
	this->DeepEcho();
}
/*}}}*/
/*FUNCTION BeamVertexElementResult::DeepEcho{{{1*/
void BeamVertexElementResult::DeepEcho(void){
		
	printf("BeamVertexElementResult:\n");
	printf("   enum: %i (%s)\n",this->enum_type,EnumAsString(this->enum_type));
	printf("   values: [%g %g]\n",this->values[0],this->values[1]);
	printf("   step: %i\n",this->step);
	printf("   time: %g\n",this->time);

}
/*}}}*/
/*FUNCTION BeamVertexElementResult::Id{{{1*/
int    BeamVertexElementResult::Id(void){ return -1; }
/*}}}*/
/*FUNCTION BeamVertexElementResult::MyRank{{{1*/
int    BeamVertexElementResult::MyRank(void){ 
	extern int my_rank;
	return my_rank; 
}
/*}}}*/
/*FUNCTION BeamVertexElementResult::Marshall{{{1*/
void  BeamVertexElementResult::Marshall(char** pmarshalled_dataset){

	char* marshalled_dataset=NULL;
	int   enum_value=0;

	/*recover marshalled_dataset: */
	marshalled_dataset=*pmarshalled_dataset;

	/*get enum value of BeamVertexElementResult: */
	enum_value=BeamVertexElementResultEnum;
	
	/*marshall enum: */
	memcpy(marshalled_dataset,&enum_value,sizeof(enum_value));marshalled_dataset+=sizeof(enum_value);
	
	/*marshall BeamVertexElementResult data: */
	memcpy(marshalled_dataset,&enum_type,sizeof(enum_type));marshalled_dataset+=sizeof(enum_type);
	memcpy(marshalled_dataset,&values,sizeof(values));marshalled_dataset+=sizeof(values);
	memcpy(marshalled_dataset,&time,sizeof(time));marshalled_dataset+=sizeof(time);
	memcpy(marshalled_dataset,&step,sizeof(step));marshalled_dataset+=sizeof(step);

	*pmarshalled_dataset=marshalled_dataset;
}
/*}}}*/
/*FUNCTION BeamVertexElementResult::MarshallSize{{{1*/
int   BeamVertexElementResult::MarshallSize(){
	
	return sizeof(values)+
		+sizeof(enum_type)
		+sizeof(time)
		+sizeof(step)
		+sizeof(int); //sizeof(int) for enum value
}
/*}}}*/
/*FUNCTION BeamVertexElementResult::Demarshall{{{1*/
void  BeamVertexElementResult::Demarshall(char** pmarshalled_dataset){

	char* marshalled_dataset=NULL;
	int   i;

	/*recover marshalled_dataset: */
	marshalled_dataset=*pmarshalled_dataset;

	/*this time, no need to get enum type, the pointer directly points to the beginning of the 
	 *object data (thanks to DataSet::Demarshall):*/
	memcpy(&enum_type,marshalled_dataset,sizeof(enum_type));marshalled_dataset+=sizeof(enum_type);
	memcpy(&values,marshalled_dataset,sizeof(values));marshalled_dataset+=sizeof(values);
	memcpy(&time,marshalled_dataset,sizeof(time));marshalled_dataset+=sizeof(time);
	memcpy(&step,marshalled_dataset,sizeof(step));marshalled_dataset+=sizeof(step);

	/*return: */
	*pmarshalled_dataset=marshalled_dataset;
	return;
}
/*}}}*/
/*FUNCTION BeamVertexElementResult::Enum{{{1*/
int BeamVertexElementResult::Enum(void){

	return BeamVertexElementResultEnum;

}
/*}}}*/
/*FUNCTION BeamVertexElementResult::copy{{{1*/
Object* BeamVertexElementResult::copy() {
	
	return new BeamVertexElementResult(this->enum_type,this->values,this->step,this->time);

}
/*}}}*/


/*ElementResult management*/
/*FUNCTION BeamVertexElementResult::EnumType{{{1*/
int BeamVertexElementResult::EnumType(void){

	return this->enum_type;

}
/*}}}*/
/*FUNCTION BeamVertexElementResult::ProcessUnits(Parameters* parameters){{{1*/
void BeamVertexElementResult::ProcessUnits(Parameters* parameters){
	
	NodalValuesUnitConversion(this->values,2,this->enum_type,parameters);

}
/*}}}*/
/*FUNCTION BeamVertexElementResult::NumberOfNodalValues(void){{{1*/
int BeamVertexElementResult::NumberOfNodalValues(void){
	return 2;
}
/*}}}*/
/*FUNCTION BeamVertexElementResult::BeamVertexElementResult::PatchFill(int row, Patch* patch){{{1*/
void BeamVertexElementResult::PatchFill(int row, Patch* patch){
	
	 /*Here, we fill the result information into the patch object. First, let's remember what is in a row 
	  * of the patch object: enum_type step time element_id interpolation vertices_ids nodal_values
	  * Here, we will supply the enum_type, step, time, interpolation and nodal_values: */
	patch->fillresultinfo(row,this->enum_type,this->step,this->time,P1Enum,this->values,2);

}
/*}}}*/
